<?php
namespace Teknisa\CTB\Controllers;

use Doctrine\ORM\EntityManager;
use Teknisa\CTB\Services\Ctb12000ManutencaoLoteService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;

class Ctb12000ManutencaoLoteController {

    protected $lancamentosService;

    public function __construct(EntityManager $entityManager, Ctb12000ManutencaoLoteService $lancamentosService) {
        $this->entityManager      = $entityManager;
        $this->lancamentosService = $lancamentosService;
    }

    /*
     * Função para buscar as partidas e os rateios de um dado lançamento.
     */
    public function checkValidDataLancamento(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();

            // Chama a função de validação.
            $this->lancamentosService->checkValidDataLancamento($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /*
     * Função para excluir um lançamento.
     */
    public function deleteLanccont(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();

            // Chama a função de validação.
            $this->lancamentosService->deleteLanccont($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /*
     * Função para buscar as partidas e os rateios de um dado lançamento.
     */
    public function getLanccont(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();

            // Busca os arrays de dados.
            $partidas = $this->lancamentosService->getPartidas($row["CDEMPRESA"], $row["NRLANCCONT"]);
            $rateios = $this->lancamentosService->getRateios($row["CDEMPRESA"], $row["NRLANCCONT"]);

            // Cria os respectivos DataSets.
            $partidasDataSet = new DataSet('partidas', $partidas);
            $rateiosDataSet = new DataSet('rateios', $rateios);

            // Adiciona os DataSets no response da requisição.
            $response->addDataSet($partidasDataSet);
            $response->addDataSet($rateiosDataSet);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /*
     * Função para salvar os dados de um lançamento.
     */
    public function saveLanclote(DTO\Request\DataSet $dataset, DTO\Response $response){
        try {
            //var_dump($dataset);die();
            $dataset = $dataset->getDataSet()->getRows();
            
            $cabecalho   = $dataset[0];
            $lancamentos = $dataset[1];
            $rateio      = $dataset[2];
            $modoAdd        = $dataset[4][0];
            
            //var_dump($modo);die();
            $this->lancamentosService->saveLanclote($cabecalho, $lancamentos, $rateio, $modoAdd);
    
            $response->addDataSet(new DataSet('saveLanclote', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteDadosLanc(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            //var_dump($row);die();

            $this->lancamentosService->deleteDadosLanc($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteTodosDadosLanc(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            //var_dump($row);die();

            $this->lancamentosService->deleteTodosDadosLanc($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
        
    
    /**
     * Rastreio Contabil
     * @param Row $row
     * @param Response $response
     */
    public function rastreioContabil(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $rows = $row->getRow();
            
            $rastreioContabil = $this->lancamentosService->rastreioContabil($rows, $response);
            
            $response->addDataSet(new DataSet('rastreioContabil', [$rastreioContabil]));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function conferirLote(DTO\Request\DataSet $dataset, DTO\Response $response){
        try {
            $dataset = $dataset->getDataSet()->getRows();
            
            $semErro = $this->lancamentosService->conferirLote($dataset);
            $response->addDataSet(new DataSet('resul', [$semErro]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function liberarLote(DTO\Request\DataSet $dataset, DTO\Response $response){
        try {
            $dataset = $dataset->getDataSet()->getRows();
            
            $this->lancamentosService->liberarLote($dataset);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function reverterLote(DTO\Request\DataSet $dataset, DTO\Response $response){
        try {
            $dataset = $dataset->getDataSet()->getRows();
            
            $this->lancamentosService->reverterLote($dataset);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}
